@@ -26,15 +26,16 @@
 using System;
 using System.Collections.Generic;
 using System.IO;
+using System.Runtime.Serialization.Formatters;
 using System.Text;
 using Newtonsoft.Json.Linq;
 using Newtonsoft.Json.Tests.TestObjects;
 #if !NETFX_CORE
 using NUnit.Framework;
 #else
-using Microsoft.VisualStudio.TestTools.UnitTesting;
-using TestFixture = Microsoft.VisualStudio.TestTools.UnitTesting.TestClassAttribute;
-using Test = Microsoft.VisualStudio.TestTools.UnitTesting.TestMethodAttribute;
+using Microsoft.VisualStudio.TestPlatform.UnitTestFramework;
+using TestFixture = Microsoft.VisualStudio.TestPlatform.UnitTestFramework.TestClassAttribute;
+using Test = Microsoft.VisualStudio.TestPlatform.UnitTestFramework.TestMethodAttribute;
 #endif
 
 namespace Newtonsoft.Json.Tests.Serialization
@@ -242,7 +243,7 @@ namespace Newtonsoft.Json.Tests.Serialization
 }";
 
       ExceptionAssert.Throws<JsonSerializationException>(
-        @"Cannot preserve reference to array or readonly list: System.String[][]. Path '$values', line 3, position 15.",
+        @"Cannot preserve reference to array or readonly list, or list created from a non-default constructor: System.String[][]. Path '$values', line 3, position 15.",
         () =>
           {
             JsonConvert.DeserializeObject<string[][]>(json,
@@ -1055,6 +1056,27 @@ namespace Newtonsoft.Json.Tests.Serialization
       Assert.IsTrue(ReferenceEquals(c2, c3));
       Assert.IsFalse(ReferenceEquals(c2, c4));
     }
+
+    [Test]
+    public void DuplicateId()
+    {
+      string json = @"{
+  ""Data"": {
+    ""Prop1"": {
+      ""$id"": ""1"",
+      ""MyProperty"": 0
+    },
+    ""Prop2"": {
+      ""$id"": ""1"",
+      ""MyProperty"": 0
+    }
+  }
+}";
+
+      ExceptionAssert.Throws<JsonSerializationException>(
+        "Error reading object reference '1'. Path 'Data.Prop2.MyProperty', line 9, position 20.",
+        () => JsonConvert.DeserializeObject<PropertyItemIsReferenceObject>(json));
+    }
   }
 
   public class PropertyItemIsReferenceBody
