@@ -25,18 +25,18 @@
 
 using System;
 using System.Collections;
-#if !(NET35 || NET20 || SILVERLIGHT || WINDOWS_PHONE || PORTABLE)
+#if !(NET35 || NET20 || SILVERLIGHT || PORTABLE || PORTABLE40)
 using System.Collections.Concurrent;
 #endif
 using System.Collections.Generic;
 using System.ComponentModel;
-#if !(NET35 || NET20 || WINDOWS_PHONE || PORTABLE)
+#if !(NET35 || NET20 || PORTABLE40)
 using System.Dynamic;
 #endif
 using System.Globalization;
 using System.Reflection;
 using System.Runtime.Serialization;
-#if !(NETFX_CORE || PORTABLE)
+#if !(NETFX_CORE || PORTABLE || PORTABLE40)
 using System.Security.Permissions;
 #endif
 using System.Xml.Serialization;
@@ -44,9 +44,6 @@ using Newtonsoft.Json.Converters;
 using Newtonsoft.Json.Utilities;
 using Newtonsoft.Json.Linq;
 using System.Runtime.CompilerServices;
-#if NETFX_CORE || PORTABLE
-using ICustomAttributeProvider = Newtonsoft.Json.Utilities.CustomAttributeProvider;
-#endif
 #if NET20
 using Newtonsoft.Json.Utilities.LinqBridge;
 #else
@@ -97,22 +94,26 @@ namespace Newtonsoft.Json.Serialization
     }
     private static readonly IList<JsonConverter> BuiltInConverters = new List<JsonConverter>
       {
-#if !(SILVERLIGHT || NET20 || NETFX_CORE || PORTABLE)
+#if !(SILVERLIGHT || NET20 || NETFX_CORE || PORTABLE40 || PORTABLE)
         new EntityKeyMemberConverter(),
 #endif
-#if !(NET35 || NET20 || WINDOWS_PHONE || PORTABLE)
+#if !(NET35 || NET20 || PORTABLE40)
         new ExpandoObjectConverter(),
 #endif
-#if (!(SILVERLIGHT || PORTABLE) || WINDOWS_PHONE)
+#if (!(SILVERLIGHT || PORTABLE40) || WINDOWS_PHONE)
         new XmlNodeConverter(),
 #endif
-#if !(SILVERLIGHT || NETFX_CORE || PORTABLE)
+#if !(SILVERLIGHT || NETFX_CORE || PORTABLE40 || PORTABLE)
         new BinaryConverter(),
         new DataSetConverter(),
         new DataTableConverter(),
+#endif
+#if NETFX_CORE
+        new JsonValueConverter(),
 #endif
         new KeyValuePairConverter(),
-        new BsonObjectIdConverter()
+        new BsonObjectIdConverter(),
+        new RegexConverter()
       };
 
     private static Dictionary<ResolverContractKey, JsonContract> _sharedContractCache;
@@ -133,7 +134,7 @@ namespace Newtonsoft.Json.Serialization
       get { return JsonTypeReflector.DynamicCodeGeneration; }
     }
 
-#if !NETFX_CORE
+#if !(NETFX_CORE || PORTABLE)
     /// <summary>
     /// Gets or sets the default members search flags.
     /// </summary>
@@ -151,7 +152,7 @@ namespace Newtonsoft.Json.Serialization
     /// </value>
     public bool SerializeCompilerGeneratedMembers { get; set; }
 
-#if !(SILVERLIGHT || NETFX_CORE || PORTABLE)
+#if !(SILVERLIGHT || NETFX_CORE || PORTABLE || PORTABLE40)
     /// <summary>
     /// Gets or sets a value indicating whether to ignore the <see cref="ISerializable"/> interface when serializing and deserializing types.
     /// </summary>
@@ -191,7 +192,7 @@ namespace Newtonsoft.Json.Serialization
 #if !NETFX_CORE
       DefaultMembersSearchFlags = BindingFlags.Public | BindingFlags.Instance;
 #endif
-#if !(SILVERLIGHT || NETFX_CORE || PORTABLE)
+#if !(SILVERLIGHT || NETFX_CORE || PORTABLE || PORTABLE40)
       IgnoreSerializableAttribute = true;
 #endif
 
@@ -256,7 +257,7 @@ namespace Newtonsoft.Json.Serialization
     protected virtual List<MemberInfo> GetSerializableMembers(Type objectType)
     {
       bool ignoreSerializableAttribute;
-#if !(SILVERLIGHT || NETFX_CORE || PORTABLE)
+#if !(SILVERLIGHT || NETFX_CORE || PORTABLE || PORTABLE40)
       ignoreSerializableAttribute = IgnoreSerializableAttribute;
 #else
       ignoreSerializableAttribute = true;
@@ -271,7 +272,7 @@ namespace Newtonsoft.Json.Serialization
       
       if (memberSerialization != MemberSerialization.Fields)
       {
-#if !PocketPC && !NET20
+#if !NET20
         DataContractAttribute dataContractAttribute = JsonTypeReflector.GetDataContractAttribute(objectType);
 #endif
 
@@ -292,10 +293,10 @@ namespace Newtonsoft.Json.Serialization
             {
               // add members that are explicitly marked with JsonProperty/DataMember attribute
               // or are a field if serializing just fields
-              if (JsonTypeReflector.GetAttribute<JsonPropertyAttribute>(member.GetCustomAttributeProvider()) != null)
+              if (JsonTypeReflector.GetAttribute<JsonPropertyAttribute>(member) != null)
                 serializableMembers.Add(member);
-#if !PocketPC && !NET20
-              else if (dataContractAttribute != null && JsonTypeReflector.GetAttribute<DataMemberAttribute>(member.GetCustomAttributeProvider()) != null)
+#if !NET20
+              else if (dataContractAttribute != null && JsonTypeReflector.GetAttribute<DataMemberAttribute>(member) != null)
                 serializableMembers.Add(member);
 #endif
               else if (memberSerialization == MemberSerialization.Fields && member.MemberType() == MemberTypes.Field)
@@ -304,7 +305,7 @@ namespace Newtonsoft.Json.Serialization
           }
         }
 
-#if !PocketPC && !SILVERLIGHT && !NET20
+#if !SILVERLIGHT && !NET20
         Type match;
         // don't include EntityKey on entities objects... this is a bit hacky
         if (objectType.AssignableToTypeName("System.Data.Objects.DataClasses.EntityObject", out match))
@@ -316,7 +317,8 @@ namespace Newtonsoft.Json.Serialization
         // serialize all fields
         foreach (MemberInfo member in allMembers)
         {
-          if (member.MemberType() == MemberTypes.Field)
+          FieldInfo field = member as FieldInfo;
+          if (field != null && !field.IsStatic)
             serializableMembers.Add(member);
         }
       }
@@ -324,7 +326,7 @@ namespace Newtonsoft.Json.Serialization
       return serializableMembers;
     }
 
-#if !PocketPC && !SILVERLIGHT && !NET20
+#if !SILVERLIGHT && !NET20
     private bool ShouldSerializeEntityMember(MemberInfo memberInfo)
     {
       PropertyInfo propertyInfo = memberInfo as PropertyInfo;
@@ -349,7 +351,7 @@ namespace Newtonsoft.Json.Serialization
       InitializeContract(contract);
 
       bool ignoreSerializableAttribute;
-#if !(SILVERLIGHT || NETFX_CORE || PORTABLE)
+#if !(SILVERLIGHT || NETFX_CORE || PORTABLE || PORTABLE40)
       ignoreSerializableAttribute = IgnoreSerializableAttribute;
 #else
       ignoreSerializableAttribute = true;
@@ -362,15 +364,22 @@ namespace Newtonsoft.Json.Serialization
       if (attribute != null)
         contract.ItemRequired = attribute._itemRequired;
 
+      ConstructorInfo overrideConstructor = GetAttributeConstructor(contract.NonNullableUnderlyingType);
+        
       // check if a JsonConstructorAttribute has been defined and use that
-      if (contract.NonNullableUnderlyingType.GetConstructors(BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic).Any(c => c.IsDefined(typeof(JsonConstructorAttribute), true)))
+      if (overrideConstructor != null)
       {
-        ConstructorInfo constructor = GetAttributeConstructor(contract.NonNullableUnderlyingType);
-        if (constructor != null)
-        {
-          contract.OverrideConstructor = constructor;
-          contract.ConstructorParameters.AddRange(CreateConstructorParameters(constructor, contract.Properties));
-        }
+        contract.OverrideConstructor = overrideConstructor;
+        contract.ConstructorParameters.AddRange(CreateConstructorParameters(overrideConstructor, contract.Properties));
+      }
+      else if (contract.MemberSerialization == MemberSerialization.Fields)
+      {
+#if !(SILVERLIGHT || NETFX_CORE || PORTABLE40 || PORTABLE)
+        // mimic DataContractSerializer behaviour when populating fields by overriding default creator to create an uninitialized object
+        // note that this is only possible when the application is fully trusted so fall back to using the default constructor (if available) in partial trust
+        if (JsonTypeReflector.FullyTrusted)
+          contract.DefaultCreator = contract.GetUninitializedObject;
+#endif
       }
       else if (contract.DefaultCreator == null || contract.DefaultCreatorNonPublic)
       {
@@ -381,9 +390,75 @@ namespace Newtonsoft.Json.Serialization
           contract.ConstructorParameters.AddRange(CreateConstructorParameters(constructor, contract.Properties));
         }
       }
+
+      contract.ExtensionDataSetter = GetExtensionDataForType(contract.NonNullableUnderlyingType);
+
       return contract;
     }
 
+    private ExtensionDataSetter GetExtensionDataForType(Type type)
+    {
+      ExtensionDataSetter extensionDataSetter = null;
+
+      foreach (Type baseType in GetClassHierarchyForType(type))
+      {
+        IList<MemberInfo> members = new List<MemberInfo>();
+        members.AddRange(baseType.GetProperties(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.DeclaredOnly));
+        members.AddRange(baseType.GetFields(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.DeclaredOnly));
+
+        foreach (MemberInfo member in members)
+        {
+          MemberTypes memberType = member.MemberType();
+          if (memberType != MemberTypes.Property && memberType != MemberTypes.Field)
+            continue;
+
+          // last instance of attribute wins on type if there are multiple
+          if (!member.IsDefined(typeof(JsonExtensionDataAttribute), false))
+            continue;
+
+          Type t = ReflectionUtils.GetMemberUnderlyingType(member);
+
+          Type dictionaryType;
+          if (ReflectionUtils.ImplementsGenericDefinition(t, typeof(IDictionary<,>), out dictionaryType))
+          {
+            Type keyType = dictionaryType.GetGenericArguments()[0];
+            Type valueType = dictionaryType.GetGenericArguments()[1];
+
+            // change type to a class if it is the base interface so it can be instantiated if needed
+            if (ReflectionUtils.IsGenericDefinition(t, typeof(IDictionary<,>)))
+              t = typeof(Dictionary<,>).MakeGenericType(keyType, valueType);
+
+            if (keyType.IsAssignableFrom(typeof(string)) && valueType.IsAssignableFrom(typeof(JToken)))
+            {
+              MethodInfo addMethod = t.GetMethod("Add", new [] { keyType, valueType });
+              Func<object, object> getExtensionDataDictionary = JsonTypeReflector.ReflectionDelegateFactory.CreateGet<object>(member);
+              Action<object, object> setExtensionDataDictionary = JsonTypeReflector.ReflectionDelegateFactory.CreateSet<object>(member);
+              Func<object> createExtensionDataDictionary = JsonTypeReflector.ReflectionDelegateFactory.CreateDefaultConstructor<object>(t);
+              MethodCall<object, object> setExtensionDataDictionaryValue = JsonTypeReflector.ReflectionDelegateFactory.CreateMethodCall<object>(addMethod);
+
+              extensionDataSetter = (o, key, value) =>
+                {
+                  object dictionary = getExtensionDataDictionary(o);
+                  if (dictionary == null)
+                  {
+                    dictionary = createExtensionDataDictionary();
+                    setExtensionDataDictionary(o, dictionary);
+                  }
+
+                  setExtensionDataDictionaryValue(dictionary, key, value);
+                };
+
+              continue;
+            }
+          }
+
+          throw new JsonException("Invalid extension data attribute on '{0}'. Member '{1}' type must implement IDictionary<string, JToken>.".FormatWith(CultureInfo.InvariantCulture, GetClrTypeFullName(member.DeclaringType), member.Name));
+        }
+      }
+
+      return extensionDataSetter;
+    }
+
     private ConstructorInfo GetAttributeConstructor(Type objectType)
     {
       IList<ConstructorInfo> markedConstructors = objectType.GetConstructors(BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic).Where(c => c.IsDefined(typeof(JsonConstructorAttribute), true)).ToList();
@@ -393,6 +468,10 @@ namespace Newtonsoft.Json.Serialization
       else if (markedConstructors.Count == 1)
         return markedConstructors[0];
 
+      // little hack to get Version objects to deserialize correctly
+      if (objectType == typeof(Version))
+        return objectType.GetConstructor(new [] { typeof(int), typeof(int), typeof(int), typeof(int) });
+
       return null;
     }
 
@@ -420,7 +499,10 @@ namespace Newtonsoft.Json.Serialization
 
       foreach (ParameterInfo parameterInfo in constructorParameters)
       {
-        JsonProperty matchingMemberProperty = memberProperties.GetClosestMatchProperty(parameterInfo.Name);
+        // it is possible to generate a ParameterInfo with a null name using Reflection.Emit
+        // protect against an ArgumentNullException from GetClosestMatchProperty by testing for null here
+        JsonProperty matchingMemberProperty = (parameterInfo.Name != null) ? memberProperties.GetClosestMatchProperty(parameterInfo.Name) : null;
+
         // type must match as well as name
         if (matchingMemberProperty != null && matchingMemberProperty.PropertyType != parameterInfo.ParameterType)
           matchingMemberProperty = null;
@@ -448,8 +530,7 @@ namespace Newtonsoft.Json.Serialization
       property.PropertyType = parameterInfo.ParameterType;
 
       bool allowNonPublicAccess;
-      bool hasExplicitAttribute;
-      SetPropertySettingsFromAttributes(property, parameterInfo.GetCustomAttributeProvider(), parameterInfo.Name, parameterInfo.Member.DeclaringType, MemberSerialization.OptOut, out allowNonPublicAccess, out hasExplicitAttribute);
+      SetPropertySettingsFromAttributes(property, parameterInfo, parameterInfo.Name, parameterInfo.Member.DeclaringType, MemberSerialization.OptOut, out allowNonPublicAccess);
 
       property.Readable = false;
       property.Writable = true;
@@ -460,7 +541,10 @@ namespace Newtonsoft.Json.Serialization
         property.PropertyName = (property.PropertyName != parameterInfo.Name) ? property.PropertyName : matchingMemberProperty.PropertyName;
         property.Converter = property.Converter ?? matchingMemberProperty.Converter;
         property.MemberConverter = property.MemberConverter ?? matchingMemberProperty.MemberConverter;
-        property.DefaultValue = property.DefaultValue ?? matchingMemberProperty.DefaultValue;
+
+        if (!property._hasExplicitDefaultValue && matchingMemberProperty._hasExplicitDefaultValue)
+          property.DefaultValue = matchingMemberProperty.DefaultValue;
+
         property._required = property._required ?? matchingMemberProperty._required;
         property.IsReference = property.IsReference ?? matchingMemberProperty.IsReference;
         property.NullValueHandling = property.NullValueHandling ?? matchingMemberProperty.NullValueHandling;
@@ -477,10 +561,10 @@ namespace Newtonsoft.Json.Serialization
     /// Resolves the default <see cref="JsonConverter" /> for the contract.
     /// </summary>
     /// <param name="objectType">Type of the object.</param>
-    /// <returns></returns>
+    /// <returns>The contract's default <see cref="JsonConverter" />.</returns>
     protected virtual JsonConverter ResolveContractConverter(Type objectType)
     {
-      return JsonTypeReflector.GetJsonConverter(objectType.GetCustomAttributeProvider(), objectType);
+      return JsonTypeReflector.GetJsonConverter(objectType, objectType);
     }
 
     private Func<object> GetDefaultCreator(Type createdType)
@@ -488,7 +572,7 @@ namespace Newtonsoft.Json.Serialization
       return JsonTypeReflector.ReflectionDelegateFactory.CreateDefaultConstructor<object>(createdType);
     }
 
-#if !PocketPC && !NET20
+#if !NET20
     [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Portability", "CA1903:UseOnlyApiFromTargetedFramework", MessageId = "System.Runtime.Serialization.DataContractAttribute.#get_IsReference()")]
 #endif
     private void InitializeContract(JsonContract contract)
@@ -498,7 +582,7 @@ namespace Newtonsoft.Json.Serialization
       {
         contract.IsReference = containerAttribute._isReference;
       }
-#if !PocketPC && !NET20
+#if !NET20
       else
       {
         DataContractAttribute dataContractAttribute = JsonTypeReflector.GetDataContractAttribute(contract.NonNullableUnderlyingType);
@@ -527,14 +611,11 @@ namespace Newtonsoft.Json.Serialization
 
     private void ResolveCallbackMethods(JsonContract contract, Type t)
     {
-      if (t.BaseType() != null)
-        ResolveCallbackMethods(contract, t.BaseType());
-
-      MethodInfo onSerializing;
-      MethodInfo onSerialized;
-      MethodInfo onDeserializing;
-      MethodInfo onDeserialized;
-      MethodInfo onError;
+      List<SerializationCallback> onSerializing;
+      List<SerializationCallback> onSerialized;
+      List<SerializationCallback> onDeserializing;
+      List<SerializationCallback> onDeserialized;
+      List<SerializationErrorCallback> onError;
 
       GetCallbackMethodsForType(t, out onSerializing, out onSerialized, out onDeserializing, out onDeserialized, out onError);
 
@@ -542,34 +623,34 @@ namespace Newtonsoft.Json.Serialization
       {
 #if NETFX_CORE
         if (!t.IsGenericType() || (t.GetGenericTypeDefinition() != typeof(ConcurrentDictionary<,>)))
-          contract.OnSerializing = onSerializing;
+          contract.OnSerializingCallbacks.AddRange(onSerializing);
 #else
-        contract.OnSerializing = onSerializing;
+        contract.OnSerializingCallbacks.AddRange(onSerializing);
 #endif
       }
 
       if (onSerialized != null)
-        contract.OnSerialized = onSerialized;
+        contract.OnSerializedCallbacks.AddRange(onSerialized);
 
       if (onDeserializing != null)
-        contract.OnDeserializing = onDeserializing;
+        contract.OnDeserializingCallbacks.AddRange(onDeserializing);
 
       if (onDeserialized != null)
       {
         // ConcurrentDictionary throws an error here so don't use its OnDeserialized - http://json.codeplex.com/discussions/257093
-#if !(NET35 || NET20 || SILVERLIGHT || WINDOWS_PHONE || PORTABLE)
+#if !(NET35 || NET20 || SILVERLIGHT || WINDOWS_PHONE || PORTABLE || PORTABLE40)
         if (!t.IsGenericType() || (t.GetGenericTypeDefinition() != typeof(ConcurrentDictionary<,>)))
-          contract.OnDeserialized = onDeserialized;
+          contract.OnDeserializedCallbacks.AddRange(onDeserialized);
 #else
-        contract.OnDeserialized = onDeserialized;
+        contract.OnDeserializedCallbacks.AddRange(onDeserialized);
 #endif
       }
 
       if (onError != null)
-        contract.OnError = onError;
+        contract.OnErrorCallbacks.AddRange(onError);
     }
 
-    private void GetCallbackMethodsForType(Type type, out MethodInfo onSerializing, out MethodInfo onSerialized, out MethodInfo onDeserializing, out MethodInfo onDeserialized, out MethodInfo onError)
+    private void GetCallbackMethodsForType(Type type, out List<SerializationCallback> onSerializing, out List<SerializationCallback> onSerialized, out List<SerializationCallback> onDeserializing, out List<SerializationCallback> onDeserialized, out List<SerializationErrorCallback> onError)
     {
       onSerializing = null;
       onSerialized = null;
@@ -577,39 +658,75 @@ namespace Newtonsoft.Json.Serialization
       onDeserialized = null;
       onError = null;
 
-      foreach (MethodInfo method in type.GetMethods(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly))
+      foreach (Type baseType in GetClassHierarchyForType(type))
       {
-        // compact framework errors when getting parameters for a generic method
-        // lame, but generic methods should not be callbacks anyway
-        if (method.ContainsGenericParameters)
-          continue;
+        // while we allow more than one OnSerialized total, only one can be defined per class
+        MethodInfo currentOnSerializing = null;
+        MethodInfo currentOnSerialized = null;
+        MethodInfo currentOnDeserializing = null;
+        MethodInfo currentOnDeserialized = null;
+        MethodInfo currentOnError = null;
 
-        Type prevAttributeType = null;
-        ParameterInfo[] parameters = method.GetParameters();
+        foreach (MethodInfo method in baseType.GetMethods(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly))
+        {
+          // compact framework errors when getting parameters for a generic method
+          // lame, but generic methods should not be callbacks anyway
+          if (method.ContainsGenericParameters)
+            continue;
 
-        if (IsValidCallback(method, parameters, typeof(OnSerializingAttribute), onSerializing, ref prevAttributeType))
-        {
-          onSerializing = method;
-        }
-        if (IsValidCallback(method, parameters, typeof(OnSerializedAttribute), onSerialized, ref prevAttributeType))
-        {
-          onSerialized = method;
-        }
-        if (IsValidCallback(method, parameters, typeof(OnDeserializingAttribute), onDeserializing, ref prevAttributeType))
-        {
-          onDeserializing = method;
-        }
-        if (IsValidCallback(method, parameters, typeof(OnDeserializedAttribute), onDeserialized, ref prevAttributeType))
-        {
-          onDeserialized = method;
-        }
-        if (IsValidCallback(method, parameters, typeof(OnErrorAttribute), onError, ref prevAttributeType))
-        {
-          onError = method;
+          Type prevAttributeType = null;
+          ParameterInfo[] parameters = method.GetParameters();
+
+          if (IsValidCallback(method, parameters, typeof(OnSerializingAttribute), currentOnSerializing, ref prevAttributeType))
+          {
+            onSerializing = onSerializing ?? new List<SerializationCallback>();
+            onSerializing.Add(JsonContract.CreateSerializationCallback(method));
+            currentOnSerializing = method;
+          }
+          if (IsValidCallback(method, parameters, typeof(OnSerializedAttribute), currentOnSerialized, ref prevAttributeType))
+          {
+            onSerialized = onSerialized ?? new List<SerializationCallback>();
+            onSerialized.Add(JsonContract.CreateSerializationCallback(method));
+            currentOnSerialized = method;
+          }
+          if (IsValidCallback(method, parameters, typeof(OnDeserializingAttribute), currentOnDeserializing, ref prevAttributeType))
+          {
+            onDeserializing = onDeserializing ?? new List<SerializationCallback>();
+            onDeserializing.Add(JsonContract.CreateSerializationCallback(method));
+            currentOnDeserializing = method;
+          }
+          if (IsValidCallback(method, parameters, typeof(OnDeserializedAttribute), currentOnDeserialized, ref prevAttributeType))
+          {
+            onDeserialized = onDeserialized ?? new List<SerializationCallback>();
+            onDeserialized.Add(JsonContract.CreateSerializationCallback(method));
+            currentOnDeserialized = method;
+          }
+          if (IsValidCallback(method, parameters, typeof(OnErrorAttribute), currentOnError, ref prevAttributeType))
+          {
+            onError = onError ?? new List<SerializationErrorCallback>();
+            onError.Add(JsonContract.CreateSerializationErrorCallback(method));
+            currentOnError = method;
+          }
         }
       }
     }
 
+    private List<Type> GetClassHierarchyForType(Type type)
+    {
+      List<Type> ret = new List<Type>();
+
+      Type current = type;
+      while (current != null && current != typeof(object))
+      {
+        ret.Add(current);
+        current = current.BaseType();
+      }
+
+      // Return the class list in order of simple => complex
+      ret.Reverse();
+      return ret;
+    }
+
     /// <summary>
     /// Creates a <see cref="JsonDictionaryContract"/> for the given type.
     /// </summary>
@@ -664,7 +781,7 @@ namespace Newtonsoft.Json.Serialization
       return contract;
     }
 
-#if !(SILVERLIGHT || NETFX_CORE || PORTABLE)
+#if !(SILVERLIGHT || NETFX_CORE || PORTABLE40 || PORTABLE)
     /// <summary>
     /// Creates a <see cref="JsonISerializableContract"/> for the given type.
     /// </summary>
@@ -687,7 +804,7 @@ namespace Newtonsoft.Json.Serialization
     }
 #endif
 
-#if !(NET35 || NET20 || WINDOWS_PHONE || PORTABLE)
+#if !(NET35 || NET20 || PORTABLE40)
     /// <summary>
     /// Creates a <see cref="JsonDynamicContract"/> for the given type.
     /// </summary>
@@ -727,7 +844,7 @@ namespace Newtonsoft.Json.Serialization
     {
       Type t = ReflectionUtils.EnsureNotNullableType(objectType);
 
-      if (JsonConvert.IsJsonPrimitiveType(t))
+      if (IsJsonPrimitiveType(objectType))
         return CreatePrimitiveContract(objectType);
 
       if (JsonTypeReflector.GetJsonObjectAttribute(t) != null)
@@ -751,22 +868,48 @@ namespace Newtonsoft.Json.Serialization
       if (CanConvertToString(t))
         return CreateStringContract(objectType);
 
-#if !(SILVERLIGHT || NETFX_CORE || PORTABLE)
+#if !(SILVERLIGHT || NETFX_CORE || PORTABLE40 || PORTABLE)
       if (!IgnoreSerializableInterface && typeof(ISerializable).IsAssignableFrom(t))
         return CreateISerializableContract(objectType);
 #endif
 
-#if !(NET35 || NET20 || WINDOWS_PHONE || PORTABLE)
+#if !(NET35 || NET20 || PORTABLE40)
       if (typeof(IDynamicMetaObjectProvider).IsAssignableFrom(t))
         return CreateDynamicContract(objectType);
 #endif
 
+#if !(PORTABLE || NETFX_CORE)
+      // tested last because it is not possible to automatically deserialize custom IConvertible types
+      if (IsIConvertible(t))
+        return CreatePrimitiveContract(t);
+#endif
+
       return CreateObjectContract(objectType);
     }
 
+    internal static bool IsJsonPrimitiveType(Type t)
+    {
+      PrimitiveTypeCode typeCode = ConvertUtils.GetTypeCode(t);
+
+      return (typeCode != PrimitiveTypeCode.Empty && typeCode != PrimitiveTypeCode.Object);
+    }
+
+#if !(PORTABLE || NETFX_CORE)
+    internal static bool IsIConvertible(Type t)
+    {
+      if (typeof(IConvertible).IsAssignableFrom(t)
+        || (ReflectionUtils.IsNullableType(t) && typeof(IConvertible).IsAssignableFrom(Nullable.GetUnderlyingType(t))))
+      {
+        return !typeof(JToken).IsAssignableFrom(t);
+      }
+
+      return false;
+    }
+#endif
+
     internal static bool CanConvertToString(Type type)
     {
-#if !(NETFX_CORE || PORTABLE)
+#if !(NETFX_CORE || PORTABLE40 || PORTABLE)
       TypeConverter converter = ConvertUtils.GetConverter(type);
 
       // use the objectType's TypeConverter if it has one and can convert to a string
@@ -785,7 +928,7 @@ namespace Newtonsoft.Json.Serialization
       if (type == typeof(Type) || type.IsSubclassOf(typeof(Type)))
         return true;
 
-#if SILVERLIGHT || PocketPC
+#if SILVERLIGHT
       if (type == typeof(Guid) || type == typeof(Uri) || type == typeof(TimeSpan))
         return true;
 #endif
@@ -870,11 +1013,13 @@ namespace Newtonsoft.Json.Serialization
       // warning - this method use to cause errors with Intellitrace. Retest in VS Ultimate after changes
       IValueProvider valueProvider;
 
-#if !(SILVERLIGHT || PORTABLE || NETFX_CORE)
+#if !(SILVERLIGHT || PORTABLE40 || PORTABLE || NETFX_CORE)
       if (DynamicCodeGeneration)
         valueProvider = new DynamicValueProvider(member);
       else
         valueProvider = new ReflectionValueProvider(member);
+#elif !(PORTABLE40)
+      valueProvider = new ExpressionValueProvider(member);
 #else
       valueProvider = new ReflectionValueProvider(member);
 #endif
@@ -896,11 +1041,19 @@ namespace Newtonsoft.Json.Serialization
       property.ValueProvider = CreateMemberValueProvider(member);
 
       bool allowNonPublicAccess;
-      bool hasExplicitAttribute;
-      SetPropertySettingsFromAttributes(property, member.GetCustomAttributeProvider(), member.Name, member.DeclaringType, memberSerialization, out allowNonPublicAccess, out hasExplicitAttribute);
+      SetPropertySettingsFromAttributes(property, member, member.Name, member.DeclaringType, memberSerialization, out allowNonPublicAccess);
 
-      property.Readable = ReflectionUtils.CanReadMemberValue(member, allowNonPublicAccess);
-      property.Writable = ReflectionUtils.CanSetMemberValue(member, allowNonPublicAccess, hasExplicitAttribute);
+      if (memberSerialization != MemberSerialization.Fields)
+      {
+        property.Readable = ReflectionUtils.CanReadMemberValue(member, allowNonPublicAccess);
+        property.Writable = ReflectionUtils.CanSetMemberValue(member, allowNonPublicAccess, property.HasMemberAttribute);
+      }
+      else
+      {
+        // write to readonly fields
+        property.Readable = true;
+        property.Writable = true;
+      }
       property.ShouldSerialize = CreateShouldSerializeTest(member);
 
       SetIsSpecifiedActions(property, member, allowNonPublicAccess);
@@ -908,19 +1061,12 @@ namespace Newtonsoft.Json.Serialization
       return property;
     }
 
-    private void SetPropertySettingsFromAttributes(JsonProperty property, ICustomAttributeProvider attributeProvider, string name, Type declaringType, MemberSerialization memberSerialization, out bool allowNonPublicAccess, out bool hasExplicitAttribute)
+    private void SetPropertySettingsFromAttributes(JsonProperty property, object attributeProvider, string name, Type declaringType, MemberSerialization memberSerialization, out bool allowNonPublicAccess)
     {
-      hasExplicitAttribute = false;
-
-#if !PocketPC && !NET20
+#if !NET20
       DataContractAttribute dataContractAttribute = JsonTypeReflector.GetDataContractAttribute(declaringType);
 
-      MemberInfo memberInfo = null;
-#if !(NETFX_CORE || PORTABLE)
-      memberInfo = attributeProvider as MemberInfo;
-#else
-      memberInfo = attributeProvider.UnderlyingObject as MemberInfo;
-#endif
+      MemberInfo memberInfo = attributeProvider as MemberInfo;
 
       DataMemberAttribute dataMemberAttribute;
       if (dataContractAttribute != null && memberInfo != null)
@@ -931,12 +1077,12 @@ namespace Newtonsoft.Json.Serialization
 
       JsonPropertyAttribute propertyAttribute = JsonTypeReflector.GetAttribute<JsonPropertyAttribute>(attributeProvider);
       if (propertyAttribute != null)
-        hasExplicitAttribute = true;
+        property.HasMemberAttribute = true;
 
       string mappedName;
       if (propertyAttribute != null && propertyAttribute.PropertyName != null)
         mappedName = propertyAttribute.PropertyName;
-#if !PocketPC && !NET20
+#if !NET20
       else if (dataMemberAttribute != null && dataMemberAttribute.Name != null)
         mappedName = dataMemberAttribute.Name;
 #endif
@@ -951,28 +1097,32 @@ namespace Newtonsoft.Json.Serialization
       {
         property._required = propertyAttribute._required;
         property.Order = propertyAttribute._order;
+        property.DefaultValueHandling = propertyAttribute._defaultValueHandling;
         hasMemberAttribute = true;
       }
-#if !PocketPC && !NET20
+#if !NET20
       else if (dataMemberAttribute != null)
       {
         property._required = (dataMemberAttribute.IsRequired) ? Required.AllowNull : Required.Default;
         property.Order = (dataMemberAttribute.Order != -1) ? (int?) dataMemberAttribute.Order : null;
+        property.DefaultValueHandling = (!dataMemberAttribute.EmitDefaultValue) ? (DefaultValueHandling?) DefaultValueHandling.Ignore : null;
         hasMemberAttribute = true;
       }
 #endif
 
       bool hasJsonIgnoreAttribute =
         JsonTypeReflector.GetAttribute<JsonIgnoreAttribute>(attributeProvider) != null
-#if !(SILVERLIGHT || NETFX_CORE || PORTABLE)
+        // automatically ignore extension data dictionary property if it is public
+        || JsonTypeReflector.GetAttribute<JsonExtensionDataAttribute>(attributeProvider) != null
+#if !(SILVERLIGHT || NETFX_CORE || PORTABLE40 || PORTABLE)
         || JsonTypeReflector.GetAttribute<NonSerializedAttribute>(attributeProvider) != null
 #endif
-        ;
+;
 
       if (memberSerialization != MemberSerialization.OptIn)
       {
-       bool hasIgnoreDataMemberAttribute = false;
-        
+        bool hasIgnoreDataMemberAttribute = false;
+
 #if !(NET20 || NET35)
         hasIgnoreDataMemberAttribute = (JsonTypeReflector.GetAttribute<IgnoreDataMemberAttribute>(attributeProvider) != null);
 #endif
@@ -992,10 +1142,10 @@ namespace Newtonsoft.Json.Serialization
       property.MemberConverter = JsonTypeReflector.GetJsonConverter(attributeProvider, property.PropertyType);
 
       DefaultValueAttribute defaultValueAttribute = JsonTypeReflector.GetAttribute<DefaultValueAttribute>(attributeProvider);
-      property.DefaultValue = (defaultValueAttribute != null) ? defaultValueAttribute.Value : null;
+      if (defaultValueAttribute != null)
+        property.DefaultValue = defaultValueAttribute.Value;
 
       property.NullValueHandling = (propertyAttribute != null) ? propertyAttribute._nullValueHandling : null;
-      property.DefaultValueHandling = (propertyAttribute != null) ? propertyAttribute._defaultValueHandling : null;
       property.ReferenceLoopHandling = (propertyAttribute != null) ? propertyAttribute._referenceLoopHandling : null;
       property.ObjectCreationHandling = (propertyAttribute != null) ? propertyAttribute._objectCreationHandling : null;
       property.TypeNameHandling = (propertyAttribute != null) ? propertyAttribute._typeNameHandling : null;
@@ -1017,11 +1167,11 @@ namespace Newtonsoft.Json.Serialization
       if (memberSerialization == MemberSerialization.Fields)
         allowNonPublicAccess = true;
 
-#if !PocketPC && !NET20
+#if !NET20
       if (dataMemberAttribute != null)
       {
         allowNonPublicAccess = true;
-        hasExplicitAttribute = true;
+        property.HasMemberAttribute = true;
       }
 #endif
     }
@@ -1080,4 +1230,4 @@ namespace Newtonsoft.Json.Serialization
       return ResolvePropertyName(propertyName);
     }
   }
-}
\ No newline at end of file
+}
